//
//		Definition of the main game class
//
#ifndef THE_MAIN_SCENE_H
#define THE_MAIN_SCENE_H

#include "theGameState.h"
#include "theStringManager.h"
#include "ldwScene.h"
#include "ldwImage.h"
#include "theWavesClass.h"
#include "theHomingClass.h"
#include "theChemicalTray.h"
#include "theToolTray.h"
class theDealerSay;
class theFloatingAnims;
class theGraphicsManager;
class thePlantManager;
class theSoundManager;
class theCollectingClass;

class ldwButton;
class ldwGameWindow;
class ldwSoundTrack;
class ldwSoundFx;
class ldwTextControl;

const int theMainScene_c_iMaxSaleNotifications = 8;
#define cCircleCorrectionY 92
#define cCircleCorrectionX 1

#define cNetToolDrawOffsetX				-5
#define cNetToolDrawOffsetY				-13
#define cNetToolClickOffsetX			-10
#define cNetToolClickOffsetY			-10

#define cScissorsToolDrawOffsetX		0
#define cScissorsToolDrawOffsetY		0
#define cScissorsToolClickOffsetX		-10
#define cScissorsToolClickOffsetY		0

#define cChemicalToolDrawOffsetX		-25
#define cChemicalToolDrawOffsetY		-15

#define cDragPotLimitLeft				172
#define cMainAreaLimitLeft				208
typedef struct 
{	
	bool  on;
	float currentOpacity;
	ldwPoint currentPosition;
	ldwPoint driftVector;
	bool waxing;
    ldwUINT32 beamOnTimer;
} BeamOfLightType;

typedef struct 
	{
		bool exists;
		int degreesIn;
		long double radiansIn;
		long double focusOutput;
		int waveRedrawTimer;
	
} waveType;
	
class theMainScene : public ldwScene
{
public:

	theMainScene();
	~theMainScene();

	bool HandleKey(int messageId,int keyCode,int character);
	const bool HandleMouseUpEvent(const ldwPoint location);
	const bool HandleMouseDownEvent(const ldwPoint location);
	bool HandleMouse(int messageId,ldwPoint location);
	bool HandleMessage(int messageId,long data);

protected:

	void Activate(bool active);
	void DrawScene();
	void DrawSceneFinal();
	void UpdateScene();

private:
	void DrawFocusCircle(ldwPoint pos);
	void UpdateFocusCircle();
	bool LDWPointInRect(int x,int y,int left,int top,int width,int height);

	void RedrawPlants();
	void DrawPlant(int xPos, int yPos, int whichPlant);
	void SelectPlant (int row, int column);
	const int SelectPlant_(const int iCursorX, const int iCursorY, int &iRow, int &iColumn);
	const int SelectSeed(const int iCursorX, const int iCursorY, int &iRow, int &iColumn);
	const int SelectTool(const int iCursorX, const int iCursorY);

	void DrawAnimations();
	void DrawLicense (int whichPlant);
	void DrawGraphicMeters (int value, int x, int y, int type);

	void DrawTools();
	static void theMainScene::ReturnSeed(int iData);
	void ReturnTool(void);

	const bool MoveSeedToSeedBox(const int iSeedTrayIndex, int &iRow, int &iColumn);
	const bool TrashCan(const int x, const int y);

	void ApplyToolOne     (PlantType * whichPlant);
	void ApplyToolTwo     (PlantType * whichPlant);
	void ApplyToolThree   (int row, int column);
	void ApplyToolFour    (PlantType * whichPlant);
	void ApplyToolOne     ();
	void ApplyToolTwo     ();
	void ApplyToolThree   ();
	void ApplyToolFour    ();
	void ConceiveSeeds    (PlantType * recipient, PlantType * donor);
	void InstantiatePlant (SeedType * whichSeed, PlantType * whichPlant);
	int  FindFreeSeedBox  ();

	int SetPlantValue     (PlantType * whichPlant);

	bool ForwardWaitingMessage(StringId whichMessage);

	void ReportPlantTimers(int row, int column);


	void UpdateLightBeams();
	void DrawLightBeams();
	void SpawnBeamOfLight();
	void InitBeamOfLight();

	class penStateNode
	{
	public:
		penStateNode(float x,float y,float angle,float scale,float fscale,int segCount,penStateNode *next) :

			mX(x),
			mY(y),
			mAngle(angle),
			mScale(scale),
			mFScale(fscale),
			mSegCount(segCount),
			mNext(next)
		{
		}

		float mX;
		float mY;
		float mAngle;
		float mScale;
		float mFScale;
		int   mSegCount;

		penStateNode *mNext;
	};

	penStateNode *penStateStack;

	void DrawLPlant(int startX,int startY,char *lPlantString,float turnAngle = 20.0f,float startScale = 1.0f,float autoScale = 0.8f,float segLength = 5.0f,float angleNoise = 0,float speedScale = 1.0f);

	void GrowLPlant(char *lPlantString,int maxLength,int depth,char *seed,char *rule1,char *rule2 = 0,char *rule3 = 0);

	void GrowLString(char targetChar,char *lSubString,char *targetString,int maxLength);

	void CalcPoint(float startX,float startY,float distance,float angle,float &endX,float &endY);

	enum {eMaxPlantStringLength = 30000};

	char lPlantString[eMaxPlantStringLength + 1];

	int plantDepth;

	bool useDepthScale;

	bool mouseIsDown;
	ldwPoint mouseDownPosition;
	unsigned int mouseUpTime;

	float mAngle;
	float mAngleStep;
	float mAngleRange;

	BeamOfLightType beamOfLight;

	ldwUINT32 secondTimer;
	ldwUINT32 mPlantGerminationTime;
	ldwUINT32 mPlantMatureAge;

	ldwUINT32 mNextAnimateTime;

	bool mAnimate;

	ldwGameWindow      *gameWindow;

	theDealerSay       *ds;
	theFloatingAnims   *fa;
	theGameState       *gs;
	theGraphicsManager *gm;
	thePlantManager    *pm;
	theSoundManager    *soundMgr;
	theStringManager   *stringMgr;
	theCollectingClass *collections;
	theHomingClass		*toolHomer;
	theHomingClass		*seedHomer;
	theHomingClass		*potHomer;
	theChemicalTray *	chemicalTray;
	theToolTray *		toolTray;

	theWavesClass		*waveGen;
	bool dragging;

	ldwButton *forsaleButton;
	ldwButton *sellAllButton;
	ldwButton *priceButton;

	ldwTextControl *harvestLabel;
	ldwTextControl *pollinLabel;
	ldwTextControl *seedsLabel;
	ldwTextControl *toolsLabel;
	ldwTextControl *trashLabel;

	ldwImage *hand;
	ldwImage *handOpen;
	ldwImage *focusCircle;
	ldwButton	   *menuButton;

	ldwButton  *lastTipButton;
	bool        mLastTipButtonShowing;

	ldwPoint		toolPos;
	ldwPoint		homingSeedPos;
	ldwPoint		tempSeedPos;
	ldwPoint		potPos;
	ldwPoint		homingPotPos;
	ldwPoint		selectedPlant;
	bool			drawFocusCircle;
	bool			isDragging;
	ldwPoint		dragStartPos;

	ldwHintControl *	m_pChemicalHintControlArray[maxChemicalTraySlots];

	typedef struct
	{
		int				m_iPrice;
	}
	SSaleNotification;

	SSaleNotification	m_SaleNotificationQueue[theMainScene_c_iMaxSaleNotifications];
	int					m_iSaleNotificationQueueHead;
	int					m_iSaleNotificationQueueTail;
	ldwUINT32			m_uiSaleNotificationTimer;

};
#endif